@@ -1,5 +1,9 @@
+import warnings
 from dataclasses import dataclass
+from dataclasses import field
+from functools import cached_property
 from typing import Any
+from typing import Dict
 from typing import Union
 
 import numpy as np
@@ -14,19 +18,26 @@ from estimagic.inference.ml_covs import cov_jacobian
 from estimagic.inference.ml_covs import cov_robust
 from estimagic.inference.ml_covs import cov_strata_robust
 from estimagic.inference.shared import calculate_ci
-from estimagic.inference.shared import calculate_inference_quantities
+from estimagic.inference.shared import calculate_estimation_summary
+from estimagic.inference.shared import calculate_free_estimates
 from estimagic.inference.shared import calculate_p_values
-from estimagic.inference.shared import check_is_optimized_and_derivative_case
+from estimagic.inference.shared import calculate_summary_data_estimation
+from estimagic.inference.shared import FreeParams
 from estimagic.inference.shared import get_derivative_case
 from estimagic.inference.shared import transform_covariance
+from estimagic.inference.shared import transform_free_cov_to_cov
+from estimagic.inference.shared import transform_free_values_to_params_tree
 from estimagic.optimization.optimize import maximize
 from estimagic.optimization.optimize_result import OptimizeResult
 from estimagic.parameters.block_trees import block_tree_to_matrix
 from estimagic.parameters.block_trees import matrix_to_block_tree
 from estimagic.parameters.conversion import Converter
 from estimagic.parameters.conversion import get_converter
+from estimagic.parameters.space_conversion import InternalParams
 from estimagic.shared.check_option_dicts import check_numdiff_options
 from estimagic.shared.check_option_dicts import check_optimization_options
+from estimagic.utilities import get_rng
+from estimagic.utilities import to_pickle
 
 
 def estimate_ml(
@@ -56,7 +67,7 @@ def estimate_ml(
     While we have good defaults, you can still configure each aspect of each step
     via the optional arguments of this function. If you find it easier to do the
     maximization separately, you can do so and just provide the optimal parameters as
-    ``params`` and set ``optimize_options=False``.
+    ``params`` and set ``optimize_options=False``
 
     Args:
         loglike (callable): Likelihood function that takes a params (and potentially
@@ -81,7 +92,7 @@ def estimate_ml(
         upper_bounds (pytree): As lower_bounds. Can be ``np.inf`` for parameters with
             no upper bound.
         constraints (list, dict): List with constraint dictionaries or single dict.
-            See .. _link: ../../docs/source/how_to_guides/how_to_use_constraints.ipynb
+            See :ref:`constraints`.
         logging (pathlib.Path, str or False): Path to sqlite3 file (which typically has
             the file extension ``.db``. If the file does not exist, it will be created.
             The dashboard can only be used when logging is used.
@@ -126,7 +137,6 @@ def estimate_ml(
     # ==================================================================================
     # Check and process inputs
     # ==================================================================================
-
     is_optimized = optimize_options is False
 
     if not is_optimized:
@@ -142,9 +152,6 @@ def estimate_ml(
     jac_case = get_derivative_case(jacobian)
     hess_case = get_derivative_case(hessian)
 
-    check_is_optimized_and_derivative_case(is_optimized, jac_case)
-    check_is_optimized_and_derivative_case(is_optimized, hess_case)
-
     check_numdiff_options(numdiff_options, "estimate_ml")
     numdiff_options = {} if numdiff_options in (None, False) else numdiff_options
     loglike_kwargs = {} if loglike_kwargs is None else loglike_kwargs
@@ -211,8 +218,7 @@ def estimate_ml(
     # Get the converter for params and function outputs
     # ==================================================================================
 
-    converter, flat_estimates = get_converter(
-        func=loglike,
+    converter, internal_estimates = get_converter(
         params=estimates,
         constraints=constraints,
         lower_bounds=lower_bounds,
@@ -229,8 +235,9 @@ def estimate_ml(
     # ==================================================================================
 
     if jac_case == "closed-form":
-        x = converter.params_to_internal(estimates)
-        int_jac = converter.derivative_to_internal(jacobian_eval, x)
+        int_jac = converter.derivative_to_internal(
+            jacobian_eval, internal_estimates.values
+        )
     elif jac_case == "numerical":
 
         def func(x):
@@ -241,9 +248,9 @@ def estimate_ml(
 
         jac_res = first_derivative(
             func=func,
-            params=flat_estimates.values,
-            lower_bounds=flat_estimates.lower_bounds,
-            upper_bounds=flat_estimates.upper_bounds,
+            params=internal_estimates.values,
+            lower_bounds=internal_estimates.lower_bounds,
+            upper_bounds=internal_estimates.upper_bounds,
             **numdiff_options,
         )
 
@@ -284,9 +291,9 @@ def estimate_ml(
 
         hess_res = second_derivative(
             func=func,
-            params=flat_estimates.values,
-            lower_bounds=flat_estimates.lower_bounds,
-            upper_bounds=flat_estimates.upper_bounds,
+            params=internal_estimates.values,
+            lower_bounds=internal_estimates.lower_bounds,
+            upper_bounds=internal_estimates.upper_bounds,
             **numdiff_options,
         )
         int_hess = hess_res["derivative"]
@@ -324,10 +331,12 @@ def estimate_ml(
     # create a LikelihoodResult object
     # ==================================================================================
 
+    free_estimates = calculate_free_estimates(estimates, internal_estimates)
+
     res = LikelihoodResult(
-        params=estimates,
+        _params=estimates,
         _converter=converter,
-        optimize_result=opt_res,
+        _optimize_result=opt_res,
         _jacobian=jacobian_eval,
         _no_jacobian_reason=_no_jac_reason,
         _hessian=hessian_eval,
@@ -335,7 +344,8 @@ def estimate_ml(
         _internal_jacobian=int_jac,
         _internal_hessian=int_hess,
         _design_info=design_info,
-        _flat_params=flat_estimates,
+        _internal_estimates=internal_estimates,
+        _free_estimates=free_estimates,
         _has_constraints=constraints not in [None, []],
     )
 
@@ -344,11 +354,14 @@ def estimate_ml(
 
 @dataclass
 class LikelihoodResult:
-    params: Any
-    _flat_params: Any
+    """Likelihood estimation results object."""
+
+    _params: Any
+    _internal_estimates: InternalParams
+    _free_estimates: FreeParams
     _converter: Converter
     _has_constraints: bool
-    optimize_result: Union[OptimizeResult, None] = None
+    _optimize_result: Union[OptimizeResult, None] = None
     _jacobian: Any = None
     _no_jacobian_reason: Union[str, None] = None
     _hessian: Any = None
@@ -356,6 +369,7 @@ class LikelihoodResult:
     _internal_jacobian: Union[np.ndarray, None] = None
     _internal_hessian: Union[np.ndarray, None] = None
     _design_info: Union[pd.DataFrame, None] = None
+    _cache: Dict = field(default_factory=dict)
 
     def __post_init__(self):
         if self._internal_jacobian is None and self._internal_hessian is None:
@@ -386,50 +400,51 @@ class LikelihoodResult:
         seed,
     ):
         if method not in self._valid_methods:
-            raise ValueError()
+            msg = f"Invalid method: {method}. Valid methods are {self._valid_methods}."
+            raise ValueError(msg)
+        args = (method, n_samples, bounds_handling, seed)
+        is_cached = args in self._cache
 
-        internal_jac = self._internal_jacobian
-        internal_hess = self._internal_hessian
-        design_info = self._design_info
-        converter = self._converter
-        flat_params = self._flat_params
-
-        if method == "jacobian":
-            int_cov = cov_jacobian(internal_jac)
-        elif method == "hessian":
-            int_cov = cov_hessian(internal_hess)
-        elif method == "robust":
-            int_cov = cov_robust(jac=internal_jac, hess=internal_hess)
-        elif method == "cluster_robust":
-            int_cov = cov_cluster_robust(
-                jac=internal_jac, hess=internal_hess, design_info=design_info
-            )
-        elif method == "strata_robust":
-            int_cov = cov_strata_robust(
-                jac=internal_jac, hess=internal_hess, design_info=design_info
-            )
+        if is_cached:
+            free_cov = self._cache[args]
         else:
-            raise ValueError(f"Invalid method: {method}")
-
-        np.random.seed(seed)
-
-        free_cov = transform_covariance(
-            flat_params=flat_params,
-            internal_cov=int_cov,
-            converter=converter,
-            n_samples=n_samples,
-            bounds_handling=bounds_handling,
-        )
+            free_cov = _calculate_free_cov_ml(
+                method=method,
+                internal_estimates=self._internal_estimates,
+                converter=self._converter,
+                internal_jacobian=self._internal_jacobian,
+                internal_hessian=self._internal_hessian,
+                n_samples=n_samples,
+                design_info=self._design_info,
+                bounds_handling=bounds_handling,
+                seed=seed,
+            )
+            if seed is not None:
+                self._cache[args] = free_cov
+            elif self._converter.has_transforming_constraints:
+                msg = (
+                    "seed is set to None and constraints are transforming. This leads "
+                    "to randomness in the result. To avoid random behavior, choose a "
+                    "non-None seed."
+                )
+                warnings.warn(msg)
 
         return free_cov
 
+    @property
+    def params(self):
+        return self._params
+
+    @property
+    def optimize_result(self):
+        return self._optimize_result
+
     @property
     def jacobian(self):
         if self._jacobian is None:
             raise NotAvailableError(
                 f"No jacobian is available because {self._no_jacobian_reason}."
             )
-
         return self._jacobian
 
     @property
@@ -438,26 +453,25 @@ class LikelihoodResult:
             raise NotAvailableError(
                 f"No hessian is available because {self._no_hessian_reason}."
             )
-
         return self._hessian
 
-    @property
+    @cached_property
     def _se(self):
         return self.se()
 
-    @property
+    @cached_property
     def _cov(self):
         return self.cov()
 
-    @property
+    @cached_property
     def _summary(self):
         return self.summary()
 
-    @property
+    @cached_property
     def _ci(self):
         return self.ci()
 
-    @property
+    @cached_property
     def _p_values(self):
         return self.p_values()
 
@@ -492,8 +506,7 @@ class LikelihoodResult:
 
         Returns:
             Any: A pytree with the same structure as params containing standard errors
-            for the parameter estimates.
-
+                for the parameter estimates.
         """
         free_cov = self._get_free_cov(
             method=method,
@@ -502,12 +515,14 @@ class LikelihoodResult:
             seed=seed,
         )
 
-        helper = np.full(len(self._flat_params.values), np.nan)
-        helper[self._flat_params.free_mask] = np.sqrt(np.diagonal(free_cov))
+        free_se = np.sqrt(np.diagonal(free_cov))
 
-        out = self._converter.params_from_internal(helper)
-
-        return out
+        se = transform_free_values_to_params_tree(
+            values=free_se,
+            free_params=self._free_estimates,
+            params=self._params,
+        )
+        return se
 
     def cov(
         self,
@@ -517,7 +532,7 @@ class LikelihoodResult:
         return_type="pytree",
         seed=None,
     ):
-        """Calculate the variance-covariance matrix of the estimated parameters.
+        """Calculate the variance-covariance (matrix) of the estimated parameters.
 
         Args:
             method (str): One of "jacobian", "hessian", "robust", "cluster_robust",
@@ -544,9 +559,8 @@ class LikelihoodResult:
                 transforming constraints.
 
         Returns:
-            Any: The covariance matrix of the estimated parameters as block-pytree or
-                numpy array.
-
+            Any: The covariance matrix of the estimated parameters as block-pytree,
+                numpy.ndarray or pandas.DataFrame.
         """
         free_cov = self._get_free_cov(
             method=method,
@@ -554,20 +568,13 @@ class LikelihoodResult:
             bounds_handling=bounds_handling,
             seed=seed,
         )
-        if return_type == "array":
-            out = free_cov
-        elif return_type == "dataframe":
-            free_index = np.array(self._flat_params.names)[self._flat_params.free_mask]
-            out = pd.DataFrame(data=free_cov, columns=free_index, index=free_index)
-        elif return_type == "pytree":
-            if len(free_cov) != len(self._flat_params.values):
-                raise NotAvailableError(
-                    "Covariance matrices in block-pytree format are only available if "
-                    "there are no constraints that reduce the number of free "
-                    "parameters."
-                )
-            out = matrix_to_block_tree(free_cov, self.params, self.params)
-        return out
+        cov = transform_free_cov_to_cov(
+            free_cov=free_cov,
+            free_params=self._free_estimates,
+            params=self._params,
+            return_type=return_type,
+        )
+        return cov
 
     def summary(
         self,
@@ -601,23 +608,22 @@ class LikelihoodResult:
             seed (int): Seed for the random number generator. Only used if there are
                 transforming constraints.
 
-
         Returns:
             Any: The estimation summary as pytree of DataFrames.
-
         """
-        free_cov = self._get_free_cov(
+        summary_data = calculate_summary_data_estimation(
+            self,
+            free_estimates=self._free_estimates,
             method=method,
+            ci_level=ci_level,
             n_samples=n_samples,
             bounds_handling=bounds_handling,
             seed=seed,
         )
-
-        summary = calculate_inference_quantities(
-            estimates=self.params,
-            flat_estimates=self._flat_params,
-            free_cov=free_cov,
-            ci_level=ci_level,
+        summary = calculate_estimation_summary(
+            summary_data=summary_data,
+            names=self._free_estimates.all_names,
+            free_names=self._free_estimates.free_names,
         )
         return summary
 
@@ -658,7 +664,6 @@ class LikelihoodResult:
                 confidence intervals.
             Any: Pytree with the same structure as params containing upper bounds of
                 confidence intervals.
-
         """
         free_cov = self._get_free_cov(
             method=method,
@@ -667,19 +672,18 @@ class LikelihoodResult:
             seed=seed,
         )
 
-        free_values = self._flat_params.values[self._flat_params.free_mask]
-        free_se = np.sqrt(np.diagonal(free_cov))
-
-        free_lower, free_upper = calculate_ci(free_values, free_se, ci_level)
-
-        helper = np.full(len(self._flat_params.values), np.nan)
-        helper[self._flat_params.free_mask] = free_lower
-        lower = self._converter.params_from_internal(helper)
-
-        helper = np.full(len(self._flat_params.values), np.nan)
-        helper[self._flat_params.free_mask] = free_upper
-        upper = self._converter.params_from_internal(helper)
+        free_lower, free_upper = calculate_ci(
+            free_values=self._free_estimates.values,
+            free_standard_errors=np.sqrt(np.diagonal(free_cov)),
+            ci_level=ci_level,
+        )
 
+        lower, upper = (
+            transform_free_values_to_params_tree(
+                values, free_params=self._free_estimates, params=self._params
+            )
+            for values in (free_lower, free_upper)
+        )
         return lower, upper
 
     def p_values(
@@ -689,7 +693,7 @@ class LikelihoodResult:
         bounds_handling="clip",
         seed=None,
     ):
-        """Calculate p_values.
+        """Calculate p-values.
 
         Args:
             method (str): One of "jacobian", "hessian", "robust", "cluster_robust",
@@ -714,11 +718,8 @@ class LikelihoodResult:
                 transforming constraints.
 
         Returns:
-            Any: Pytree with the same structure as params containing lower bounds of
-                confidence intervals.
-            Any: Pytree with the same structure as params containing upper bounds of
-                confidence intervals.
-
+            Any: Pytree with the same structure as params containing p-values.
+            Any: Pytree with the same structure as params containing p-values.
         """
         free_cov = self._get_free_cov(
             method=method,
@@ -727,13 +728,59 @@ class LikelihoodResult:
             seed=seed,
         )
 
-        free_values = self._flat_params.values[self._flat_params.free_mask]
-        free_se = np.sqrt(np.diagonal(free_cov))
+        free_p_values = calculate_p_values(
+            free_values=self._free_estimates.values,
+            free_standard_errors=np.sqrt(np.diagonal(free_cov)),
+        )
 
-        free_p_values = calculate_p_values(free_values, free_se)
+        p_values = transform_free_values_to_params_tree(
+            free_p_values, free_params=self._free_estimates, params=self._params
+        )
+        return p_values
 
-        helper = np.full(len(self._flat_params.values), np.nan)
-        helper[self._flat_params.free_mask] = free_p_values
-        out = self._converter.params_from_internal(helper)
+    def to_pickle(self, path):
+        """Save the LikelihoodResult object to pickle.
 
-        return out
+        Args:
+            path (str, pathlib.Path): A str or pathlib.path ending in .pkl or .pickle.
+        """
+        to_pickle(self, path=path)
+
+
+def _calculate_free_cov_ml(
+    method,
+    internal_estimates,
+    converter,
+    internal_jacobian,
+    internal_hessian,
+    n_samples,
+    design_info,
+    bounds_handling,
+    seed,
+):
+    if method == "jacobian":
+        int_cov = cov_jacobian(internal_jacobian)
+    elif method == "hessian":
+        int_cov = cov_hessian(internal_hessian)
+    elif method == "robust":
+        int_cov = cov_robust(jac=internal_jacobian, hess=internal_hessian)
+    elif method == "cluster_robust":
+        int_cov = cov_cluster_robust(
+            jac=internal_jacobian, hess=internal_hessian, design_info=design_info
+        )
+    elif method == "strata_robust":
+        int_cov = cov_strata_robust(
+            jac=internal_jacobian, hess=internal_hessian, design_info=design_info
+        )
+
+    rng = get_rng(seed)
+
+    free_cov = transform_covariance(
+        internal_params=internal_estimates,
+        internal_cov=int_cov,
+        converter=converter,
+        rng=rng,
+        n_samples=n_samples,
+        bounds_handling=bounds_handling,
+    )
+    return free_cov
