@@ -39,7 +39,7 @@ def _get_max_f_changes(critvals):
     best_val = critvals[-1]
     worst_val = critvals[0]
 
-    max_change_abs = np.abs((best_val - worst_val))
+    max_change_abs = np.abs(best_val - worst_val)
     denom = max(np.abs(best_val), 0.1)
 
     max_change_rel = max_change_abs / denom
