@@ -9,6 +9,7 @@ The following arguments are not supported as ``algo_options``:
   and algorithm.
 
 """
+import contextlib
 import warnings
 
 import numpy as np
@@ -390,14 +391,10 @@ def _process_nag_result(nag_result_obj, len_x):
         processed["solution_x"] = nag_result_obj.x
     else:
         processed["solution_x"] = np.array([np.nan] * len_x)
-    try:
+    with contextlib.suppress(AttributeError):
         processed["solution_derivative"] = nag_result_obj.gradient
-    except AttributeError:
-        pass
-    try:
+    with contextlib.suppress(AttributeError):
         processed["solution_hessian"] = nag_result_obj.hessian
-    except AttributeError:
-        pass
     if processed["message"].startswith("Error (bad input)"):
         raise ValueError(processed["message"])
     return processed
