@@ -121,9 +121,7 @@ def _process_multistart_info(info, converter, primary_key, fixed_kwargs, skip_ch
 
     direction = fixed_kwargs["direction"]
 
-    starts = []
-    for x in info["start_parameters"]:
-        starts.append(converter.params_from_internal(x))
+    starts = [converter.params_from_internal(x) for x in info["start_parameters"]]
 
     optima = []
     for res, start in zip(info["local_optima"], starts):
@@ -139,16 +137,12 @@ def _process_multistart_info(info, converter, primary_key, fixed_kwargs, skip_ch
         )
         optima.append(processed)
 
-    sample = []
-    for x in info["exploration_sample"]:
-        sample.append(converter.params_from_internal(x))
+    sample = [converter.params_from_internal(x) for x in info["exploration_sample"]]
 
     if direction == "minimize":
         exploration_res = info["exploration_results"]
     else:
-        exploration_res = []
-        for res in info["exploration_results"]:
-            exploration_res.append(switch_sign(res))
+        exploration_res = [switch_sign(res) for res in info["exploration_results"]]
 
     out = {
         "start_parameters": starts,
